; File: sonar40.asm
; 4 channel sonar for ROBOR mini sumo robot 
; Developed on Mandrake Linux 7.2
; Written for 16F876 PIC microcontroller  with 5.0 MHZ clock
; Version 1.0
;
; This file can be assembled with gpasm avaliable at:
;         http://gpasm.sourceforge.net/
; The Microchip PIC assembler may also work.
;
; The hardware schematic is in sonar-2.sch
;
; Copyright Dale A. Heatherington June 2001
;
;;----------------------------------------------------------------------
;;This program is free software; you can redistribute it and/or modify  
;;it under the terms of the GNU General Public License as published by      
;;the Free Software Foundation; either version 2 of the License, or         
;;(at your option) any later version.                                       
;;
;;This program is distributed in the hope that it will be useful,       
;;but WITHOUT ANY WARRANTY; without even the implied warranty of            
;;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             
;;GNU General Public License for more details.                              
                                                                          
;;You should have received a copy of the GNU General Public License     
;;along with this program; if not, write to the Free Software               
;;Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
;;------------------------------------------------------------------------  
;
;
;   PIC16F876 28 pin DIP pin assignments
;
; 2     RA0    Left Echo input  (analog)
; 3     RA1    Right Echo input (analog) 
; 4     RA2    Sensitivity control (analog) 
; 5     RA3     
; 6     RA4     
; 7     RA5     
;
; 21    RB0     40 khz Piezo sonar transmitter common
; 22    RB1     Left Front sonar transmitter
; 23    RB2     Right Front sonar transmitter
; 24    RB3     Left Rear
; 25    RB4     Right Rear
; 26    RB5     IR_ENABLE for tilt/direction sensor  (output)
; 27    RB6     Tilt      (Low = tilted)    input
; 28    RB7     Direction (Low = forward)   input
;
; 11    RC0     Left Indicator LED    (low true)
; 12    RC1     Right Indicator LED   (low true)
; 13    RC2     Analog mux address line 0 (A) 0=front  1=rear
; 14    RC3     SPI clock output
; 15    RC4     SPI chip select output, LOW TRUE
; 16    RC5     SPI data output
; 17    RC6     
; 18    RC7     
;
;-----------------------------------------------------------------------
; SPI Data format
;
; The SPI port is "output only".  11 bytes are transmitted in each frame.
; 
; SPI Port byte sequence.
;
; 1     Flag, 7E hex, to verify framing at receiver
; 2     byte count excluding flag byte  (11)
; 3     type = 02 = 40 khz sonar
; 4     Left front echo time
; 5     Left front echo amplitude
; 6     Right front echo time
; 7     Right front echo amplitude
; 8     Left rear echo time
; 9     Left rear echo amplitude
;10     Right rear echo time
;11     Right rear echo amplitude
;12     Sensitivity pot value
;13     Tilt/Direction sensor bits

;
;All echo distances are 100 counts per meter or 1 per cm.
;The minimum distance is 16 cm  and the max is 1.5 meters.
;A ZERO reading means 0 to 16 cm .
;A 150 reading means 1.5 meters or more.

;------------------------------------------------------------- 
  


	RADIX	        dec
	processor	16F876


	__CONFIG h'3d72'    ;    
        
 
        include "p16f876.inc"




 
;------------------------------------------------------------

        


;SPI port bits (bit bang)
;PORTC:
SPI_CS          equ     4
SPI_CLK         equ     3
SPI_DOUT        equ     5



;----------------------------------
;40 khz microphone addressing bit on PORTC
; 0 = front
; 1 = rear


MIC_0           equ     2
MUTE            equ     6
LED_LEFT        equ     0
LED_RIGHT       equ     1


;PORTA
blanking        equ     4       ;RA4 is blanking low true

;-------------------------------

;Define how many cycles of 40khz are in one pulse
N_cycles        equ     7

;
;define our variables
;
;
j_main          equ     20h
k_main          equ     21h
i_main          equ     22h

t_echo_LF       equ     23h     ;echo round trip time, Left Front
a_echo_LF       equ     24h     ;echo peak amplitude, Left Front

t_echo_RF       equ     25h
a_echo_RF       equ     26h

t_echo_LR       equ     27h
a_echo_LR       equ     28h

t_echo_RR       equ     29h
a_echo_RR       equ     2ah

t_echo_L        equ     30h     ; Left echo time accumulator
t_echo_R        equ     31h
a_echo_L        equ     32h     ; Left echo amplitude accumulator
a_echo_R        equ     33h

a_temp          equ     34h

a_temp_L        equ     34h     ;reused...
a_temp_R        equ     35h
t_max           equ     35h     ;reused
a_max           equ     36h

t_temp          equ     37h
spi_data        equ     38h

workb           equ     3eh
temp            equ     3fh
av_ctr          equ     40h

left_ptr        equ     41h
right_ptr       equ     42h


pot             equ     43h     ;sensitivity pot setting
pot_2           equ     44h
pot_4           equ     45h
pot_8           equ     46h
pot_16          equ     47h
pot_32          equ     48h

battery         equ     50h
tilt_dir        equ     51h

D_SIZE          equ     20                      ;Size of data arrays

data_lf         equ     0a0h                    ;20 byte array for left front amplitude data
data_rf         equ     data_lf+D_SIZE          ;20 byte array for right front amplitude data
data_lr         equ     data_rf+D_SIZE          ;left rear
data_rr         equ     data_lr+D_SIZE          ;right rear


;--------------------------------------------------



;    MACROS

;---------------------------------------------------------------------------

;Piezo ultrasonic transducer differencal driver macro.  For 5 mhz cpu clock.
;
;The common lead is toggled 180 degrees out of phase with the signal lead
;to make a 10 volt difference across the transducer.
;
;Creates one cycle of 40 khz (less 2.4uS for overhead in other code)
;                                   
cycle_pz        MACRO

        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
	nop
	nop
        nop
        movlw   1fh
        xorwf   workb,W    ;Flip bits 0..2  result to W
        movwf   PORTB      ;put in PORTB and diddle the piezo transducers
        nop
        nop
        nop
        nop
        nop
        nop
       	nop
        nop
        nop
        nop
        nop
        nop
        nop
	xorlw   1fh          ;flip them back
        movwf   PORTB
        endm

;---------------------------------------------------------------------

;Actual code starts here...



	ORG 0

main_code

	clrf    STATUS
	movlw   0ffh
	movwf   PORTB
        movwf   PORTA
        movwf   PORTC
        clrf    STATUS
        bsf     STATUS,RP0      ;bank 1
        
        movlw   0efh            ;Port A all inputs
	movwf   TRISA           ; except RA4
         
        movlw   0c0h             ;Port B all outputs except bits 6 and 7
        movwf   TRISB
        
        movlw   000h             ;port c all outputs
        movwf   TRISC
        
        movlw   80h             ;PORTB pullups disabled
        movwf    OPTION_REG
        
        movlw   0               ;
        movwf   ADCON1          ;PORTA inputs are analog
       
        bcf     STATUS,RP0      ;bank 0
        clrf    INTCON          ;no interrupts
        clrf    ADCON0          ;A to D converter OFF
        
        bsf     PORTB,SPI_CLK   ;SPI clock high
        goto    label_0001
        
;
t_table addwf   PCL,f
        retlw   24          ;0..3    0..7 cm
        retlw   18          ;4..7    8..15 cm
        retlw   12          ;8..11   16..23 cm
        retlw   9           ;12..15  24..31 cm
        retlw   6           ;16..19  32..39 cm
        retlw   4           ;20..23  40..47 cm
        retlw   3           ;24..27  48..55 cm
        retlw   2           ;28..31  56..63 cm
        
        
        
get_threshold_old
       
        movwf   temp            ;Assume distance is in W
        rrf     temp,f          
        rrf     temp,f          ;and divide it by 4
        movlw   7
        andwf   temp,W          ;restrict table index 0..7
        call    t_table
        addwf   pot,W
        return                  ;return pot + table value in W
        
 
get_threshold                   ;Enter with distance in W
        movwf   temp            ;save distance in temp
        rrf     temp,f
        rrf     temp,f          ;divide by 4
        movlw   7
        andwf   temp,f          ;keep only low 3 bits
        
                                ;build a table
        bcf     STATUS,C
        rrf     pot,W           ;W = pot / 2 
        movwf   pot_2
        bcf     STATUS,C
        rrf     pot_2,W         ;W = pot / 4
        movwf   pot_4
        bcf     STATUS,C
        rrf     pot_4,W         ;W = pot / 8
        movwf   pot_8
        bcf     STATUS,C
        rrf     pot_8,W         ;W = pot / 16
        movwf   pot_16
        bcf     STATUS,C
        rrf     pot_16,W        ;W = pot / 32
        movwf   pot_32
              
        movlw   pot             ;address of pot value table to W
        addwf   temp,W          ;add the distance to create table index pointer
        movwf   FSR             ;move index to FSR
        movlw   2               ;minimum threshold is 2
        addwf    INDF,W         ;get lookup table value + 2  to W
        return                  ;return the threshold value  
        




label_0001                      ;Top of main loop

        call    read_pot        ;read the sensitivity pop value

;
;----------- Emit pulse on FRONT 40khz emitters -------

        movlw   D_SIZE * 4
        movwf   av_ctr          
        movlw   data_lf
        movwf   FSR
        
clr_dat clrf    INDF                    ;clear all sonar amplitude arrays
        incf    FSR,f
        decfsz  av_ctr,f
        goto    clr_dat                 ;Loop 80 times
        
        movlw   8
        movwf   av_ctr                  ;prepare to take 8 sonar readings for averaging
                
pulse_1
               
        movlw   N_cycles        ;
        movwf   i_main          ; Loop N_cycles times for N_cycles cycles of 40 khz
                                ;
        bcf     PORTA,blanking
        bcf     PORTC,MIC_0     ;select front receiver tranducers
        bsf     PORTC,MUTE       
          
        clrf    workb
        bsf     workb,1         ;Enable transmitter transducer 1 
        bsf     workb,2         ;and 2
        
pulse_loop_1

        cycle_pz                ;Do a 40khz cycle on piezo transducers
        decfsz  i_main,f
        goto    pulse_loop_1    ;loop N_cycles


        call    delay_blank     ;Wait for blanking interval
        ;call    delay_500
        bcf     PORTC,MUTE
        bsf     PORTA,blanking
        call    delay_200       ;wait 300us without blanking
        call    delay_100

        movlw   data_lf        ;point to arrays to hold echos
        movwf   left_ptr
        movlw   data_rf
        movwf   right_ptr
        call    get_echos      ;accumulate echo data in left and right front arrays
        
        call    delay_1000
        call    delay_1000
        call    delay_1000
        call    delay_1000
        call    delay_1000
        call    delay_1000       ;wait for distant echos to disipate
      
                 
;-------- Emit pulse on REAR 40khz emitters  ---------

pulse_2 
  
        movlw   N_cycles        ;
        movwf   i_main          ; Loop N_cycles times for N_cycles cycles of 40 khz
                                ;
       
        bcf     PORTA,blanking
        bsf     PORTC,MIC_0     ;select rear receiver transducers
        bsf     PORTC,MUTE
        
        clrf    workb
        bsf     workb,3         ;Enable transducer #3
        bsf     workb,4         ;and #4
pulse_loop_2

        cycle_pz                ;Do a 40khz cycle on piezo transducer
        decfsz  i_main,f
        goto    pulse_loop_2    ;loop N_cycles
        
       
        call    delay_blank     ;Wait for blanking interval
        
        bcf     PORTC,MUTE
        bsf     PORTA,blanking
        call    delay_200       ;wait 300us without blanking
        call    delay_100
        
        movlw   data_lr         ;set pointers to arrays
        movwf   left_ptr
        movlw   data_rr
        movwf   right_ptr
        call    get_echos       ;accumulate echo data in left and right rear arrays
        call    delay_1000
        call    delay_1000
        call    delay_1000
        call    delay_1000
        call    delay_1000
        call    delay_1000       ;wait for distant echos to disipate


        decfsz  av_ctr,f
        goto    pulse_1         ;loop 8 times to get average of 8 pings in arrays
        

        call    find_peaks      ;scan the arrays and find the max echo amplitudes
                                ;Fall into SPI port output routine below.
                                

        call    get_battery     ;read the battery voltage/3
        call    get_tilt_dir    ;read the tilt and direction sensors
       
;-------------------------------------------------------------------



;Output  bytes to SPI port

        bcf     PORTC,SPI_CS    ;Lower SPI CS line
        nop                     ;wait 7 uS
        nop
        nop
        nop
        nop
        nop
        nop
        movlw   7eh
        call    spi_out         ;Send out a flag byte
        movlw   13
        call    spi_out         ;Send byte count
        movlw   2
        call    spi_out         ;Send type indicator (2)
 ;
        movf    t_echo_LF,W     ;Left front echo time to W
        call    spi_out         ;send
        movf    a_echo_LF,W     ;right front echo amplitude W
        call    spi_out         ;send
        movf    t_echo_RF,W     ;Right Front  echo time to W
        call    spi_out         ;send
        movf    a_echo_RF,W     ;Right Front echo amplitude to W
        call    spi_out         ;send
        movf    t_echo_LR,W     ;Left Rear echo time to W
        call    spi_out         ;send
        movf    a_echo_LR,W     ;Left Rear echo amplitude to W
        call    spi_out         ;send
        movf    t_echo_RR,W     ;Right rear echo time to W
        call    spi_out         ;send
        movf    a_echo_RR,W     ;Right rear echo amplitude to W
        call    spi_out         ;send
        movf    pot,W
        call    spi_out         ;send sensitivity pot value
        movf    tilt_dir,W      ;get tilt/direction bits
        call    spi_out
        movf    battery,W       ;get battery voltage/3
        call    spi_out
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        bsf     PORTC,SPI_CS    ;raise SPI CS line


        goto label_0001         ;Infinite main loop


;------------------------------
;
;Bit bang the SPI port...
;
spi_out movwf   spi_data
        movlw   8
        movwf   i_main
spi_dat bcf     PORTC,SPI_CLK   ;clock low
        bcf     PORTC,SPI_DOUT   ;clear output bit
        btfsc   spi_data,7
        bsf     PORTC,SPI_DOUT  ;set output bit if data is ONE
        rlf     spi_data,f      ;Left shift data
        nop
        nop
        bsf     PORTC,SPI_CLK   ;clock high   DATA VALID NOW
       
        nop
        nop
        nop
        nop
        nop

        decfsz  i_main,f
        goto    spi_dat         ;loop 8 times

        bcf     PORTC,SPI_DOUT  ;data low
        return
;-------------------------------------------
;
;This is the echo time and amplitude  measuring routine.
;Sample every 120us for 20mm  resolution. 
;We wait ?? ms for all returning echos.  The one with the highest amplitude is returned.
;A return time value of 0 means the distance is less than  16cm (approx).
;There are 50 counts per meter.

;***CHECK THIS FOR 120uS loop timing .
get_echos
        movlw   D_SIZE
        movwf   j_main          ;main loop limit counter
        clrf    k_main
             
e_loop0 bsf     PORTB,7
        movlw   41h             ;select A-D channel 0 (left echo)
        movwf   ADCON0
        call    delay_ADs       ;Delay A-D settling time  14uS
        bsf     ADCON0,2        ;Start A-D conversion
        
        movf    left_ptr,W      ;get left array base address
        addwf   k_main,W        ;point to array element
        movwf   FSR             ;move address to pointer register

          
wc_1    btfsc   ADCON0,2        ;wait for A-D completion
        goto    wc_1    
    
        
        movf    ADRESH,W        ;Get A-D result of left echo amplitude
        movwf   a_temp_L        ;save it
        rrf     a_temp_L,f      ;divide by 8
        rrf     a_temp_L,f
        rrf     a_temp_L,f
        movlw   1fh
        andwf   a_temp_L,W      ;final divide by 8 data result in W
        addwf   INDF,f          ;INDF = INDF + (a_temp_L / 8)
           
               
e_loop2 bcf     PORTB,7
        movlw   49h             ;select A-D channel 1 (right echo)
        movwf   ADCON0
        call    delay_ADs       ;Delay A-D settling time  14uS
        bsf     ADCON0,2        ;Start A-D conversion

        movf    right_ptr,W     ;get right array base address
        addwf   k_main,W        ;add index to point to array element
        movwf   FSR             ;move address to pointer register
                        
wc_2    btfsc   ADCON0,2        ;wait for A-D completion
        goto    wc_2

        movf    ADRESH,W        ;Get A-D result of right echo amplitude
        movwf   a_temp_R        ;save it
        rrf     a_temp_R,f        ;divide by 8
        rrf     a_temp_R,f
        rrf     a_temp_R,f
        movlw   1fh
        andwf   a_temp_R,W      ;final divide by 8 data result in W
        addwf   INDF,f          ;INDF = INDF + (a_temp_R / 8)
       
               
e_loop4 call    delay_ADs        ;trim loop out to 120uS
        call    delay_ADs
        nop
        nop
        nop
        nop
        nop
        nop
        
e_loop6
        incf    k_main,f        ;Add another 120 uS to time accumulator
        decfsz  j_main,f
        goto    e_loop0         ;loop 20 times
        return
        



;----------------------------------------
;Find max amplitude values in each of the 4 data arrays
;Place max value with time index in final target variables
;a_echo_* and t_echo_*
;
find_peaks

        clrf    a_echo_LF
        clrf    a_echo_RF
        clrf    a_echo_LR
        clrf    a_echo_RR
        
        movlw   255
        movwf   t_echo_LF
        movwf   t_echo_RF
        movwf   t_echo_LR
        movwf   t_echo_RR
        
        movlw   data_lf
        movwf   FSR             ;point to left front data array
        movlw   D_SIZE
        movwf   av_ctr          ;load counter with array size
        clrf    a_max
        clrf    t_temp

;Do Left front           
fp_1    movf    INDF,W          ;get a data value to W
        subwf   a_max,W         ;W = a_max - data
        btfss   STATUS,C
        call    save_new_max    ;save new value as a_max if it's larger than the last a_max
        incf    FSR,f           ;point to next data value
        incf    t_temp,f
        decfsz  av_ctr,f
        goto    fp_1            ;loop 20 times  for left front array
        
        movf    t_max,W
        call    get_threshold
        subwf   a_max,W         ;W = a_max - threshold
        btfss   STATUS,C
        goto    do_rf             ;skip saving if a_max is below threshold
        movf    a_max,W
        movwf   a_echo_LF 
        movf    t_max,W
        movwf   t_echo_LF

;Do Right front        
do_rf   movlw   data_rf
        movwf   FSR             ;point to right front data array
        movlw   D_SIZE
        movwf   av_ctr          ;load counter with array size
        clrf    a_max
        clrf    t_temp
         
fp_2    movf    INDF,W          ;get a data value to W
        subwf   a_max,W         ;W = a_max - data
        btfss   STATUS,C
        call    save_new_max    ;save new value as a_max if it's larger than the last a_max
        incf    FSR,f           ;point to next data value
        incf    t_temp,f
        decfsz  av_ctr,f
        goto    fp_2            ;loop 20 times  for right front array
        
        movf    t_max,W
        call    get_threshold
        subwf   a_max,W         ;W = a_max - threshold
        btfss   STATUS,C
        goto    do_lr           ;skip saving if a_max is below threshold
        movf    a_max,W
        movwf   a_echo_RF 
        movf    t_max,W
        movwf   t_echo_RF
        
;Do left rear
do_lr   movlw   data_lr
        movwf   FSR             ;point to left rear data array
        movlw   D_SIZE
        movwf   av_ctr          ;load counter with array size
        clrf    a_max
        clrf    t_temp
         
fp_3    movf    INDF,W          ;get a data value to W
        subwf   a_max,W         ;W = a_max - data
        btfss   STATUS,C
        call    save_new_max    ;save new value as a_max if it's larger than the last a_max
        incf    FSR,f           ;point to next data value
        incf    t_temp,f
        decfsz  av_ctr,f
        goto    fp_3            ;loop 20 times  for left rear array
        
        movf    t_max,W
        call    get_threshold
        subwf   a_max,W         ;W = a_max - threshold
        btfss   STATUS,C
        goto    do_rr           ;skip saving if a_max is below threshold
        movf    a_max,W
        movwf   a_echo_LR 
        movf    t_max,W
        movwf   t_echo_LR

;Do right rear        
do_rr   movlw   data_rr
        movwf   FSR             ;point to right rear data array
        movlw   D_SIZE
        movwf   av_ctr          ;load counter with array size
        clrf    a_max
        clrf    t_temp
         
fp_4    movf    INDF,W          ;get a data value to W
        subwf   a_max,W         ;W = a_max - data
        btfss   STATUS,C
        call    save_new_max    ;save new value as a_max if it's larger than the last a_max
        incf    FSR,f           ;point to next data value
        incf    t_temp,f
        decfsz  av_ctr,f
        goto    fp_4            ;loop 20 times  for right rear array
        
        movf    t_max,W
        call    get_threshold
        subwf   a_max,W         ;W = a_max - threshold
        btfss   STATUS,C
        goto    fp_done        ;skip saving if a_max is below threshold
        movf    a_max,W
        movwf   a_echo_RR 
        movf    t_max,W
        movwf   t_echo_RR  
fp_done        
        call    do_LEDs         ;set indicator LEDs        
        return                  ;All done
        
save_new_max
        movf    INDF,W         ;get data value
        movwf   a_max          ;save amplitude in a_max
        movf    t_temp,W
        movwf   t_max          ;save time value too
        
        return
        
;----------------------------------------

get_tilt_dir
        bsf     PORTB,5         ;Turn on the IR sensor LEDs
        call    delay_ADs       ;wait for sensor to settle
        movf    PORTB,W         ;get the bits 
        andlw   0c0h            ;keep only bits 6 and 7
        movwf   tilt_dir        ;save in tilt_dir
        bcf     PORTB,5         ;turn off IR 
        return
                
;----------------------------------------

get_battery
        movlw   61h             ;Select A-D channel 4 (Battery voltage/3 )
        movwf   ADCON0
        call    delay_ADs       ;Delay A-D settling time  14uS
        call    delay_ADs       ;again for total of  28 us
        bsf     ADCON0,2        ;Start A-D conversion
cb_wait_conv
        btfsc   ADCON0,2        ;Wait for completion
        goto    cb_wait_conv
        movf    ADRESH,W        ;Get A-D result to W
        movwf   battery         ;save it in battery memory location
        return


;----------------------------------------
;Reads the sensitivity adjustment control pot.
;
read_pot
        movlw   51h             ;select A-D channel 2 (sensitivity pot)
        movwf   ADCON0
        call    delay_ADs       ;Delay A-D settling time  14uS
        call    delay_ADs       ;again for total of  28 us
        bsf     ADCON0,2        ;Start A-D conversion
rp_wait_conv
        btfsc   ADCON0,2        ;Wait for completion
        goto    rp_wait_conv
        movf    ADRESH,W        ;Get A-D result to W
        movwf   pot             ;save it in pot memory location
        return

;------------------------------------------
        



do_LEDs bcf     STATUS,C
        rrf     a_echo_LF,W     
        movwf   a_echo_L
        bcf     STATUS,C
        rrf     a_echo_LR,W
        addwf   a_echo_L,f      ;average of left front and left rear
        
        bcf     STATUS,C
        rrf     a_echo_RF,W
        movwf   a_echo_R
        bcf     STATUS,C
        rrf     a_echo_RR,W
        addwf   a_echo_R,f      ;average of right front and right rear

        movf    a_echo_R,W
        iorwf   a_echo_L,W
        btfsc   STATUS,Z        ;see if both left and right amplitudes are zero
        goto    both_zero       ;return if they are
        
        movf    a_echo_R,W
        subwf   a_echo_L,W      ; L - R = W
        movwf   temp
        btfss   STATUS,C        ;test carry bit
        goto    r_greater
        bcf     PORTC,LED_LEFT  ;Left LED ON
        bsf     PORTC,LED_RIGHT ;Right LED OFF
        return
r_greater
        bcf     PORTC,LED_RIGHT ;Right LED ON
        bsf     PORTC,LED_LEFT  ;Left LED OFF
        return
        
both_zero
        bsf     PORTC,LED_LEFT  ;Left LED OFF
        bsf     PORTC,LED_RIGHT ;Right LED OFF
        return
        
        
        
        
;----------------------------------------
;
;Some time wasting delay routines.  Not all are used.
;Timed for 5 mhz cpu clock.    Assumne 3.2 uS in call and return.

delay_ADs               ;Wait for A-D converter to settle to 8 bit accuracy
       nop               ; 14 uS
       nop
       nop
       nop
       nop
       nop
       nop
       nop
       nop
       nop
       nop
       return
        
                
delay_blank         ;About 500us delay
      movlw   102
      movwf   i_main
_DBL  nop
      nop
      nop
      decfsz  i_main,f
      goto    _DBL
        
      return


;Delay 1000us        
delay_1000
        movlw   207
        movwf   i_main
_D1000L  nop
        nop
        nop
        decfsz  i_main,f
        goto    _D1000L
        
        return


;Delay 500us
delay_500
        movlw   154
        movwf   i_main
_D500L  nop
        decfsz  i_main,f
        goto    _D500L
        nop
        nop
        return


;Delay 200 us
delay_200
        movlw   61
        movwf   i_main
_D200L  nop
        decfsz  i_main,f
        goto    _D200L
        nop
        nop
        return

;Delay 100 us
delay_100
        movlw   30
        movwf   i_main
_D100L  nop
        decfsz  i_main,f
        goto    _D100L
        nop
        nop
        return


	END
